@@ -1,4 +1,4 @@
-/**
+/*
  * Licensed to the Apache Software Foundation (ASF) under one
  * or more contributor license agreements.  See the NOTICE file
  * distributed with this work for additional information
@@ -16,6 +16,7 @@
  * specific language governing permissions and limitations
  * under the License.
  */
+
 package org.apache.curator.framework.recipes.locks;
 
 import com.google.common.base.Predicate;
@@ -113,21 +114,11 @@ public class InterProcessReadWriteLock
     {
         public WriteLock(CuratorFramework client, String basePath, byte[] lockData)
         {
-            super(client, basePath, WRITE_LOCK_NAME, lockData, 1, new SortingLockInternalsDriver() {
-                @Override
-                public PredicateResults getsTheLock(
-                    CuratorFramework client,
-                    List<String> children,
-                    String sequenceNodeName,
-                    int maxLeases
-                ) throws Exception {
-                    return super.getsTheLock(client, children, sequenceNodeName, maxLeases);
-                }
-            });
+            super(client, basePath, WRITE_LOCK_NAME, lockData, 1, new SortingLockInternalsDriver());
         }
 
         @Override
-        protected String getLockPath()
+        public String getLockPath()
         {
             return super.getLockPath();
         }
@@ -137,6 +128,15 @@ public class InterProcessReadWriteLock
         public ReadLock(CuratorFramework client, String basePath, byte[] lockData, WriteLock writeLock)
         {
             super(client, basePath, READ_LOCK_NAME, lockData, Integer.MAX_VALUE, new SortingLockInternalsDriver() {
+                @Override
+                protected String getSortingSequence() {
+                    String writePath = writeLock.getLockPath();
+                    if (writePath != null) {
+                        return fixForSorting(writePath, WRITE_LOCK_NAME);
+                    }
+                    return null;
+                }
+
                 @Override
                 public PredicateResults getsTheLock(
                     CuratorFramework client,
@@ -172,7 +172,7 @@ public class InterProcessReadWriteLock
         }
 
         @Override
-        protected String getLockPath()
+        public String getLockPath()
         {
             return super.getLockPath();
         }
