@@ -1,4 +1,4 @@
-/**
+/*
  * Licensed to the Apache Software Foundation (ASF) under one
  * or more contributor license agreements.  See the NOTICE file
  * distributed with this work for additional information
@@ -23,6 +23,7 @@ import static org.junit.jupiter.api.Assertions.assertArrayEquals;
 import static org.junit.jupiter.api.Assertions.assertEquals;
 import static org.junit.jupiter.api.Assertions.assertFalse;
 import static org.junit.jupiter.api.Assertions.assertNotNull;
+import static org.junit.jupiter.api.Assertions.assertThrows;
 import static org.junit.jupiter.api.Assertions.assertTrue;
 import static org.junit.jupiter.api.Assertions.fail;
 import com.google.common.collect.Lists;
@@ -45,6 +46,7 @@ import java.util.concurrent.ExecutorService;
 import java.util.concurrent.Executors;
 import java.util.concurrent.Future;
 import java.util.concurrent.TimeUnit;
+import java.util.concurrent.TimeoutException;
 import java.util.concurrent.atomic.AtomicInteger;
 
 public class TestInterProcessReadWriteLock extends BaseClassForTests
@@ -239,6 +241,47 @@ public class TestInterProcessReadWriteLock extends BaseClassForTests
         }
     }
 
+    @Test
+    public void testContendingDowngrading() throws Exception
+    {
+        CuratorFramework client = CuratorFrameworkFactory.newClient(server.getConnectString(), new RetryOneTime(1));
+        ExecutorService executor = Executors.newCachedThreadPool();
+        try
+        {
+            client.start();
+
+            InterProcessReadWriteLock lock1 = new InterProcessReadWriteLock(client, "/lock");
+            lock1.writeLock().acquire();
+
+            CountDownLatch ready = new CountDownLatch(1);
+            Future<?> writeAcquire = executor.submit(() -> {
+                ready.countDown();
+                InterProcessReadWriteLock lock2 = new InterProcessReadWriteLock(client, "/lock");
+                lock2.writeLock().acquire();
+                fail("expect no acquire");
+                return null;
+            });
+
+            ready.await();
+            // Let lock2 have chance to do write-acquire before downgrading.
+            Thread.sleep(20);
+
+            assertTrue(lock1.readLock().acquire(5, TimeUnit.SECONDS));
+
+            lock1.writeLock().release();
+            // We still hold read lock, other write-acquire should block.
+            assertThrows(TimeoutException.class, () -> {
+                // Let lock2 have chance to respond to write-release
+                writeAcquire.get(20, TimeUnit.MILLISECONDS);
+            });
+        }
+        finally
+        {
+            TestCleanState.closeAndTestClean(client);
+            executor.shutdown();
+        }
+    }
+
     @Test
     public void testBasic() throws Exception
     {
@@ -366,79 +409,6 @@ public class TestInterProcessReadWriteLock extends BaseClassForTests
         }
     }
 
-    public static class LockPathInterProcessReadWriteLock extends InterProcessReadWriteLock
-    {
-        private final WriteLock writeLock;
-        private final ReadLock readLock;
-
-        public LockPathInterProcessReadWriteLock(CuratorFramework client, String basePath)
-        {
-            this(client, basePath, null);
-        }
-
-        public LockPathInterProcessReadWriteLock(CuratorFramework client, String basePath, byte[] lockData)
-        {
-            this(client, basePath, lockData, new WriteLock(client, basePath, lockData));
-        }
-
-        private LockPathInterProcessReadWriteLock(
-            CuratorFramework client,
-            String basePath,
-            byte[] lockData,
-            WriteLock writeLock
-        )
-        {
-            this(writeLock, new ReadLock(client, basePath, lockData, writeLock));
-        }
-
-        private LockPathInterProcessReadWriteLock(WriteLock writeLock, ReadLock readLock)
-        {
-            super(writeLock, readLock);
-            this.writeLock = writeLock;
-            this.readLock = readLock;
-        }
-
-        @Override
-        public WriteLock writeLock()
-        {
-            return writeLock;
-        }
-
-        @Override
-        public ReadLock readLock()
-        {
-            return readLock;
-        }
-
-        public static class WriteLock extends InterProcessReadWriteLock.WriteLock
-        {
-            private WriteLock(CuratorFramework client, String basePath, byte[] lockData)
-            {
-                super(client, basePath, lockData);
-            }
-
-            @Override
-            public String getLockPath()
-            {
-                return super.getLockPath();
-            }
-        }
-
-        public static class ReadLock extends InterProcessReadWriteLock.ReadLock
-        {
-            private ReadLock(CuratorFramework client, String basePath, byte[] lockData, WriteLock writeLock)
-            {
-                super(client, basePath, lockData, writeLock);
-            }
-
-            @Override
-            public String getLockPath()
-            {
-                return super.getLockPath();
-            }
-        }
-    }
-
     @Test
     public void testLockPath() throws Exception
     {
@@ -448,8 +418,8 @@ public class TestInterProcessReadWriteLock extends BaseClassForTests
         {
             client1.start();
             client2.start();
-            LockPathInterProcessReadWriteLock lock1 = new LockPathInterProcessReadWriteLock(client1, "/lock");
-            LockPathInterProcessReadWriteLock lock2 = new LockPathInterProcessReadWriteLock(client2, "/lock");
+            InterProcessReadWriteLock lock1 = new InterProcessReadWriteLock(client1, "/lock");
+            InterProcessReadWriteLock lock2 = new InterProcessReadWriteLock(client2, "/lock");
             lock1.writeLock().acquire();
             KillSession.kill(client1.getZookeeperClient().getZooKeeper());
             lock2.readLock().acquire();
