@@ -1,4 +1,4 @@
-/**
+/*
  * Licensed to the Apache Software Foundation (ASF) under one
  * or more contributor license agreements.  See the NOTICE file
  * distributed with this work for additional information
@@ -16,18 +16,16 @@
  * specific language governing permissions and limitations
  * under the License.
  */
+
 package org.apache.curator.test;
 
 import java.io.Closeable;
-import org.apache.zookeeper.server.quorum.QuorumPeerConfig;
 
-public interface ZooKeeperMainFace extends Closeable
+interface ZooKeeperMainFace extends Closeable
 {
-    void configure(QuorumConfigBuilder config, int instanceIndex) throws Exception;
-
-    void start();
+    void start(QuorumPeerConfigBuilder configBuilder);
 
     void kill();
 
-    QuorumPeerConfig getConfig() throws Exception;
+    int getClientPort() throws Exception;
 }
